import unittest

from dynatrace_extension import ActivationConfig, ActivationType


class TestActivation(unittest.TestCase):
    def test_remote_activation(self):
        raw_activation = {
            "enabled": True,
            "description": "RabbitMQ Monitoring",
            "version": "1.0.10",
            "pythonRemote": {"endpoints": [{"url": "http://127.0.0.1:15672", "user": "guest", "password": "guest"}]},
        }
        activation = ActivationConfig(raw_activation)

        self.assertTrue(activation.enabled)
        self.assertEqual(activation.description, "RabbitMQ Monitoring")
        self.assertEqual(activation.version, "1.0.10")
        self.assertEqual(activation.type, ActivationType.REMOTE)
        self.assertEqual(activation["endpoints"][0]["url"], "http://127.0.0.1:15672")
        self.assertEqual(activation.remote["endpoints"][0]["url"], "http://127.0.0.1:15672")
        self.assertFalse(activation.local)

    def test_local_activation(self):
        raw_activation = {
            "enabled": False,
            "description": "RabbitMQ Monitoring",
            "version": "1.0.10",
            "pythonLocal": {"url": "http://127.0.0.1:15672", "user": "guest", "password": "guest"},
        }
        activation = ActivationConfig(raw_activation)

        self.assertFalse(activation.enabled)
        self.assertEqual(activation.description, "RabbitMQ Monitoring")
        self.assertEqual(activation.version, "1.0.10")
        self.assertEqual(activation.type, ActivationType.LOCAL)
        self.assertEqual(activation["url"], "http://127.0.0.1:15672")
        self.assertEqual(activation.local["url"], "http://127.0.0.1:15672")
        self.assertFalse(activation.remote)

    def test_bad_key(self):
        raw_activation = {
            "enabled": False,
            "description": "RabbitMQ Monitoring",
            "version": "1.0.10",
            "pythonLocal": {"url": "http://127.0.0.1:15672", "user": "guest", "password": "guest"},
        }
        activation = ActivationConfig(raw_activation)

        with self.assertRaises(KeyError):
            _ = activation["bad_key"]

    def test_direct_access_versus_get(self):
        raw_activation = {
            "enabled": False,
            "description": "RabbitMQ Monitoring",
            "version": "1.0.10",
            "pythonLocal": {"url": "http://127.0.0.1:15672", "user": "guest", "password": "guest"},
        }
        activation = ActivationConfig(raw_activation)
        assert activation["url"] == activation.get("url", "")



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
from datetime import datetime

sys.path.insert(0, os.path.abspath("."))


# -- Project information -----------------------------------------------------
project = "dt-sdk"
copyright = f"2023–{datetime.now().year}, Dynatrace LLC"
author = "Dynatrace"

# Short version
version = "1.0.0"
# The full version, including alpha/beta/rc tags
release = version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = "en"


# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.napoleon",
    "sphinx.ext.viewcode",
    "sphinx.ext.autosectionlabel",
    "sphinx_wagtail_theme",
    "sphinxcontrib.programoutput",
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ["_templates"]

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# True to prefix each section label with the name of the document it is in,
# followed by a colon. For example, index:Introduction for a section called
# Introduction that appears in document index.rst. Useful for avoiding
# ambiguity when the same section heading appears in different documents.
autosectionlabel_prefix_document = True

# -- Autodoc -----------------------------------------------------------------

autodoc_default_options = {"member-order": "bysource", "undoc-members": True}

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = "sphinx_wagtail_theme"
html_favicon = "_static/favicon.ico"
html_show_copyright = True

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ["_static"]

# html_sidebars = {"**": ["search-field.html", "sidebar-nav-bs.html"]}

html_theme_options = {
    "project_name": "dt-sdk",
    "logo": "dt-sdk-logo.png",
    "logo_alt": "dt-sdk",
    "logo_url": "/",
    "github_url": "https://github.com/dynatrace-extensions/dt-extensions-python-sdk/tree/main/docs/",
    "footer_links": ",".join(
        [
            "Dynatrace|https://dynatrace.com/",
            "Extensions Team|mailto:extenions@dynatrace.com",
        ]
    ),
}



from __future__ import annotations

import json
from pathlib import Path

import yaml


class ExtensionYaml:
    def __init__(self, yaml_file: Path):
        self._file = yaml_file
        self._data = yaml.safe_load(yaml_file.read_text())

    @property
    def name(self) -> str:
        return self._data.get("name", "")

    @property
    def version(self) -> str:
        return self._data.get("version", "")

    @property
    def min_dynatrace_version(self) -> str:
        return self._data.get("minDynatraceVersion", "")

    @property
    def author(self) -> Author:
        return Author(self._data.get("author", {}))

    @property
    def python(self) -> Python:
        return Python(self._data.get("python", {}))

    def validate(self):
        """
        Checks that the files under 'python.activation' exist and are valid json files
        """
        if self.python.activation.remote and self.python.activation.remote.path:
            self._validate_json_file(self.python.activation.remote.path)

        if self.python.activation.local and self.python.activation.local.path:
            self._validate_json_file(self.python.activation.local.path)

    def _validate_json_file(self, raw_path: str):
        path = Path(Path(self._file).parent / raw_path)
        if not path.exists():
            msg = f"Extension yaml validation failed, file {path} does not exist"
            raise ValueError(msg)

        # Parse the file to make sure it is valid json
        with path.open() as f:
            json.load(f)

    def zip_file_name(self) -> str:
        return f"{self.name.replace(':', '_')}-{self.version}.zip"


class Python:
    def __init__(self, data: dict):
        self._data = data

    @property
    def runtime(self) -> Runtime:
        return Runtime(self._data.get("runtime", {}))

    @property
    def activation(self):
        return Activation(self._data.get("activation", {}))


class Runtime:
    def __init__(self, data: dict):
        self._data = data

    @property
    def module(self) -> str:
        return self._data.get("module", "datasourcepy")

    @property
    def version(self) -> Version:
        return Version(self._data.get("version", {}))


class Version:
    def __init__(self, data: dict):
        self._data = data

    @property
    def min_version(self) -> str:
        return self._data.get("min", "")

    @property
    def max_version(self) -> str:
        return self._data.get("max", "")


class Activation:
    def __init__(self, data: dict):
        self._data = data

    @property
    def remote(self) -> ActivationInstance | None:
        if data := self._data.get("remote"):
            return ActivationInstance(data)
        return None

    @property
    def local(self) -> ActivationInstance | None:
        if data := self._data.get("local"):
            return ActivationInstance(data)
        return None


class ActivationInstance:
    def __init__(self, data: dict):
        self._data = data

    @property
    def path(self) -> str:
        return self._data.get("path", "")


class Author:
    def __init__(self, _data: dict):
        self._data = _data

    @property
    def name(self) -> str:
        return self._data.get("name", "")



import json
import time

events = []
for i in range(5000):
    attributes = {}
    for j in range(150):
        attributes[f"attribute{j}"] = j
    events.append(attributes)

s = time.time()
print(len(json.dumps(events).encode("utf-8")))
print("json", time.time() - s)

s2 = time.time()
print(len(f"{events}".encode()))
print("f-string", time.time() - s2)
